#!/bin/bash
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# ===| BASH Stub |=============================================================

# The beginning of this script is both valid shell and valid python,
# such that the script starts with the shell and is reexecuted with
# the right python.
'''echo' $0: Starting up...
if BINOC_PYTHON_PATH="$(which python2.7 2>/dev/null)"; then
    exec $BINOC_PYTHON_PATH $0 "$@"
else
    echo "$0 error: Python 2.7 was not found on this system"
    exit 1
fi
'''

# =============================================================================

# ===| Imports |===============================================================

from __future__ import print_function
from collections import OrderedDict
from glob import glob
from time import sleep
import hashlib
import os
import json
import time

# =============================================================================

# ===| Functions |=============================================================

def funcChecksum(filename, block_size=65536):
  sha256 = hashlib.sha256()
  with open(filename, 'rb') as f:
    for block in iter(lambda: f.read(block_size), b''):
      sha256.update(block)
  return sha256.hexdigest()

def funcReadJson(aJsonFile):
  with open(aJsonFile) as f:
    jsonData = json.load(f)
  return(jsonData)

def funcGenJson(aValue):
  return json.dumps(aValue, sort_keys=True, ensure_ascii=False, indent=2)

# =============================================================================

# ===| Main |=================================================================

CURRENT_PATH = os.getcwd()
RELEASE_MANIFEST = '.build.manifest'

# Deal with existing files
if os.path.isfile(CURRENT_PATH + '/' + RELEASE_MANIFEST):
  sleep(2)
  os.remove(CURRENT_PATH + '/' + RELEASE_MANIFEST)

# Get all the json files
FINAL_MANIFEST = {
  "linux-x86_64-gtk2" : {},
  "linux-x86_64-gtk3" : {},
  "win32" : {},
  "win64" : {},
  "Linux_x86_64-gcc3-gtk2" : {},
  "Linux_x86_64-gcc3-gtk3" : {},
  "WINNT_x86-msvc" : {},
  "WINNT_x86_64-msvc" : {}
}

for _jsonFile in glob(CURRENT_PATH + '/*.json'):
  _mozJson = funcReadJson(_jsonFile)
  _displayVersion = _mozJson['moz_app_version']

  _platform = _mozJson['moz_pkg_platform']

  FINAL_MANIFEST[_platform]['meta'] = {
    "version": _mozJson['moz_app_version'],
    "displayVersion": _displayVersion,
    "buildid": _mozJson['buildid']
  }

  _fileBasename = '{0}/{1}-{2}.{3}'.format(CURRENT_PATH, 
                                           _mozJson['moz_app_name'],
                                           _mozJson['moz_app_version'],
                                           _mozJson['moz_pkg_platform'])
  for _file in glob(_fileBasename + '*'):
    if not _file.endswith('.json'):
      if _file.endswith(".tar.bz2") or _file.endswith(".tar.xz") or _file.endswith(".zip") or _file.endswith(".7z"):
        FINAL_MANIFEST[_platform]['archive'] = {
          "file": os.path.basename(_file),
          "size": os.path.getsize(_file),
          "sha256": funcChecksum(_file),
        }
      elif _file.endswith(".exe"):
        FINAL_MANIFEST[_platform]["installer"] = {
          "file": os.path.basename(_file),
          "size": os.path.getsize(_file),
          "sha256": funcChecksum(_file),
        }
      elif _file.endswith(".mar"):
        FINAL_MANIFEST[_platform]["update"] = {
          "file": os.path.basename(_file),
          "size": os.path.getsize(_file),
          "sha256": funcChecksum(_file),
        }

if FINAL_MANIFEST["win64"] == {}:
  FINAL_MANIFEST["linux-x86_64-gtk2"] = FINAL_MANIFEST["Linux_x86_64-gcc3-gtk2"]
  FINAL_MANIFEST["linux-x86_64-gtk3"] = FINAL_MANIFEST["Linux_x86_64-gcc3-gtk3"]
  FINAL_MANIFEST["win32"] = FINAL_MANIFEST["WINNT_x86-msvc"]
  FINAL_MANIFEST["win64"] = FINAL_MANIFEST["WINNT_x86_64-msvc"]

FINAL_MANIFEST = funcGenJson(FINAL_MANIFEST)
fileManifest = open(CURRENT_PATH + '/' + RELEASE_MANIFEST, 'w', 0)
fileManifest.write(FINAL_MANIFEST)
fileManifest.close()
print(FINAL_MANIFEST)

# =============================================================================